#!/bin/sh
#---+---<1>---+---<2>---+---<3>---+---<4>---+---<5>---+---<6>---+---<7>|
#
#      cplprg.sh : FPX version 1.0 : revision level 0 : 05-04-2002 :
#
#      Bourne shell script for compiling FPX programs.
#
#      Adapted 19/3/2002 D L Foulis, from ~/work03/cpl_dommino
#
   VER=1.0
   REVLVL=0
#      Test if a non-default version required.
   if test -n "$2"
   then
      VER="$2"
   fi
   if test -n "$3"
   then
      REVLVL="$3"
      if test $REVLVL -lt 0
      then
         echo "%FPX:CPLPRG.SH-FAIL, negative revision level"
         exit 1
      fi
   fi
   VEREXT=`echo v$VER | sed -e 's/\.//'`
   VERMAJ=`echo $VER | cut -f 1 -d .`
   if test $VERMAJ -lt 1
   then
      echo "%FPX:CPLPRG.SH-FAIL, major version number impossible"
      exit 1
   fi
   VERMIN=`echo $VER | cut -f 2 -d .`
   if test $VERMAJ -lt 1
   then
      echo "%FPX:CPLPRG.SH-FAIL, negative minor version number"
      exit 1
      fi
   echo "Compilation in version $VER (revision level $REVLVL) of FPX:"
#      Loop through all program names.
   for FILE in `cat program_list.txt`
   do
      PRGNAM=`echo $FILE | cut -f 2 -d /`
      DIRNAM=`echo $FILE | cut -f 1 -d /`
      if [ $1 = "all" ] || [ $1 = $PRGNAM ]
      then
         OUTDIR=bin
         NMLINK=" -O -o $PRGNAM "
         TYP=
         FEXT=f
         ETLINK=$5
#      Set particular characteristics for individual program.
         case $PRGNAM in
            as2nag)
               TYP=".o"
               NMLINK=" -c "
               OUTDIR=aux ;;
            molsym)
               FEXT=fdd ;;
            mlpshx)
               FEXT=fdd ;;
            inpshx)
               ETLINK=" aux/as2nag.o "$ETLINK
               FEXT=fdd ;;
            eneshx)
               FEXT=fdd ;;
            intmsh)
               FEXT=fdd ;;
            cntshx)
               FEXT=fdd ;;
         esac
#      Find the source corresponding to the version and revision level.
         CTMJVR=$VERMAJ
         CTMNVR=$VERMIN
         CTRVLV=$REVLVL
         SEARCH=yes
         PFOUND=no
         while [ $SEARCH = yes ]
         do
            SRCFIL=$FILE.v$CTMJVR$CTMNVR.$FEXT.$CTRVLV
            if [ -f $SRCFIL ]
            then
               SEARCH=no
               PFOUND=yes
               cp $SRCFIL tmp/$PRGNAM.$FEXT
               chmod 700 tmp/$PRGNAM.$FEXT
#      Check if code is dummy dimensioned and needs editing.
               if [ $FEXT = "fdd" ]
               then
                  SEDFIL=$FILE.sed
                  if [ -f tmp/$PRGNAM.sed ]
                  then
                     SEDFIL=tmp/$PRGNAM.sed
                  fi
                  sed -f $SEDFIL tmp/$PRGNAM.$FEXT > tmp/$PRGNAM.f
               fi
#      Do the compilation.
               echo "Compiling $PRGNAM v$CTMJVR.$CTMNVR rl$CTRVLV"
               OPSYST=`uname`
               if [ $OPSYST = "OSF1" ]
               then
                  f77 -tune host -fpe2 -nowarn $NMLINK \
                     -align dcommon tmp/$PRGNAM.f $ETLINK
               elif [ $OPSYST = "Linux" ]
               then
                  ETLINK=$ETLINK" -fno-automatic -fugly-assumed"
                  ETLINK=$ETLINK" -finit-local-zero"
                  g77 -w -fcase-upper -malign-double $NMLINK \
                     tmp/$PRGNAM.f $ETLINK 2> tmp/$PRGNAM-cpl-err.txt
#      Further marginal speed improvements if we use:
#      "-funroll-loops" and "-fexpensive-optimizations"
#      Some extra switches are necessary to compile programs
#      Note that "-fcase-upper" seems to be a crucial switch.
#      "-fno-automatic" seems to repair lack of SAVE statements.
#      "-fugly-assumed" allows arrays dimensioned as A(N,1) etc.
#      "-fbounds-check" can be passed in $5 to check initial runs.
               elif [ $OPSYST = "SunOS" ]
               then
                  f77 -w $NMLINK -dalign tmp/$PRGNAM.f $ETLINK
#      In this case we get a lot of output from the FORTRAN preprocessor
#      as it saves intermediate files in /tmp.
               else
                  ERRMES="unrecognized operating system"
                  echo "%FPX:CPLPRG.SH-FAIL, "$ERRMES
                  exit 1
               fi
               if [ "$4" != keep ]
               then
                  rm -rf tmp/$PRGNAM.f tmp/$PRGNAM-cpl-err.txt
                  rm -rf tmp/$PRGNAM.fdd
               fi
               mv $PRGNAM$TYP $OUTDIR
            fi
#      Decrement counts and check limit.
            CTRVLV=`expr $CTRVLV - 1`
            if [ $CTRVLV -lt 0 ]
            then
               CTRVLV=9
               CTMNVR=`expr $CTMNVR - 1`
               if [ $CTMNVR -lt 0 ]
               then
                  CTMNVR=9
                  CTMJVR=`expr $CTMJVR - 1`
                  if [ $CTMJVR -lt 1 ]
                  then
                     SEARCH=no
                  fi
               fi
            fi
         done
         if [ $PFOUND = no ]
         then
            ERRMES="no program $PRGNAM at or below v$VER rl$REVLVL"
            echo "%FPX:CPLPRG.SH-WARN, "$ERRMES
         fi
      fi
   done
   exit 0
