#!/bin/sh
#---+---<1>---+---<2>---+---<3>---+---<4>---+---<5>---+---<6>---+---<7>|
#
#      runprg.sh : FPX version 1.0 : revision level 0 : 05-04-2002 :
#
#      Bourne shell script for running the FPX programs.
#
#      Adapted 19/3/2002 D L Foulis, from ~/work03/runprg.
#
#---+---<1>---+---<2>---+---<3>---+---<4>---+---<5>---+---<6>---+---<7>|
#
#     Direct flow to run a particular program.
#
   case $1 in
#
#        Run INCLEM to generate Clementi-Roetti atomic data.
#
      inc)
         set -x
         ln -s wvefns/wc$2.dat      fort.1
         ln -s data1/fpx_pj2-$2.dat fort.2
         bin/inclem
         set +x
         rm fort.1 fort.2 ;;
#
#        Run CLEM to generate atomic densities and potentials.
#
      clm)
         set -x
         ln -s data1/fpx_pcm-$2.dat fort.1
         ln -s data1/fpx_pc4-$2.dat fort.4
         ln -s data2/fpx_pc7-$2.dat fort.7
         bin/clem
         set +x
         rm fort.1 fort.4 fort.7 ;;
#
#        Run HERM to generate atomic densities and potentials.
#
      hrm)
         set -x
         ln -s wvefns/wh$2.dat      fort.1
         ln -s data1/fpx_ph2-$2.dat fort.2
         bin/herm
         set +x
         rm fort.1 fort.2 ;;
#
#        Run MPMSGP to generate atomic position files.
#
      mgp)
         set -x
         ln -s data1/fpx_mm1-$2.dat  fort.1
         ln -s data1/fpx_ps2-$2.dat  fort.2
         ln -s data1/fpx_ms4-$2.dat  fort.3
         ln -s gendat/fpx_mm4-$3.dat fort.4
         bin/mpmsgp
         set +x
         rm fort.1 fort.2 fort.3 fort.4 ;;
#
#        Run DIMMSM to generate dimensions for MOLSYM.
#
      dms)
         set -x
         ln -s tmp/dms.tmp fort.1
         bin/dimmsm
         set +x
         rm fort.1
         cat tmp/dms.tmp | tr -d [:blank:] > tmp/molsym.sed
         rm tmp/dms.tmp ;;
#
#        Run MOLSYM to generate symmetry data..
#
      msm)
         set -x
         ln -s data1/fpx_ms4-$2.dat  fort.4
         ln -s data2/fpx_ms7-$2.dat  fort.7
         ln -s data2/fpx_ms9-$2.dat  fort.9
         bin/molsym
         set +x
         rm fort.4 fort.7 fort.9 ;;
#
#        Run MLPSHX to generate molecular potential multipole
#        expansions.
#
      mlp)
         set -x
         ln -s data1/fpx_pcm-$2.dat  fort.1
         ln -s data1/fpx_ps2-$2.dat  fort.2
         ln -s data2/fpx_ps3-$2.dat  fort.3
         ln -s data1/fpx_ps4-$2.dat  fort.4
         ln -s data2/fpx_ps7-$2.dat  fort.7
         ln -s data2/fpx_ps8-$2.dat  fort.8
         ln -s data2/fpx_ps9-$2.dat  fort.9
         ln -s data2/fpx_pss-$2.dat  fort.10
         ln -s data2/fpx_psd-$2.dat  fort.11
         bin/mlpshx
         set +x
         rm fort.1 fort.2 fort.4
         rm fort.3 fort.7 fort.8 fort.9 fort.10 fort.11 ;;
#
#        Run INPSHX to tidy molecular potential multipole
#        expansions.
#
      inp)
         set -x
         ln -s data1/fpx_pi1-$2.dat  fort.1
         ln -s data2/fpx_pi2-$2.dat  fort.2
         ln -s data2/fpx_ps3-$2.dat  fort.3
         ln -s data1/fpx_ps4-$2.dat  fort.4
         ln -s data2/fpx_pi7-$2.dat  fort.7
         ln -s data2/fpx_pi8-$2.dat  fort.8
         ln -s data2/fpx_pi9-$2.dat  fort.9
         bin/inpshx
         set +x
         rm fort.1 fort.3 fort.4
         rm fort.2 fort.7 fort.8 fort.9 ;;
#
#        Run ENESHX for bound-state eigenvalue search.
#
      ene)
         set -x
         ln -s data1/fpx_a01-$2.dat  fort.1
         ln -s data1/fpx_a02-$2.dat  fort.2
         ln -s data1/fpx_a03-$2.dat  fort.3
         ln -s data1/fpx_a04-$2.dat  fort.4
         ln -s data2/fpx_a07-$2.dat  fort.7
         ln -s data2/fpx_a08-$2.dat  fort.8
         ln -s data2/fpx_a09-$2.dat  fort.9
         ln -s data1/fpx_a10-$2.dat  fort.10
         bin/eneshx
         set +x
         rm fort.1 fort.2 fort.3 fort.4
         rm fort.7 fort.8 fort.9 fort.10 ;;
#
#        Run INTMSH for bound-state eigenvalue search.
#
      int)
         set -x
         ln -s data1/fpx_i01-$2.dat  fort.1
         ln -s data2/fpx_i02-$2.dat  fort.2
         bin/intmsh
         set +x
         rm fort.1 fort.2 ;;
#
#        Run CNTSHX for continuum cross-section calculations.
#
      cnt)
         set -x
         ln -s data1/fpx_b01-$2.dat  fort.1
         ln -s data1/fpx_b02-$2.dat  fort.2
         ln -s data1/fpx_b03-$2.dat  fort.3
         ln -s data1/fpx_b04-$2.dat  fort.4
         ln -s data2/fpx_b07-$2.dat  fort.7
         ln -s data2/fpx_b08-$2.dat  fort.8
         bin/cntshx
         set +x
         rm fort.1 fort.2 fort.3 fort.4
         rm fort.7 fort.8 ;;
#
#        Output the help mnemonics.
#
      help)
         more <<!
!        This command file runs the FPX programs.
!        Parameters used are:
!        <program> <p1> <p2>      <p3>     <p4>   <p5>
!         inclem    inc  C.-R.at.
!         herm      hrm  H.-S.at.
!         clem      clm  mol.id.
!         mpmsgp    mgp  mol.id.   group
!         dimmsm    dms
!         molsym    msm  mol.id.
!         mlpshx    mlp  mol.id.
!         inpshx    inp  mol.id.
!         eneshx    ene  mol.id.
!         intmsh    int  mol.id.
!         cntshx    cnt  mol.id.
!        where C.-R.at.=[accc]=Clementi-Roetti atom,
!              H.-S.at.=[accc]=Herman-Skillman atom,
!              mol.id.=[c...]=identifier of molecule,
!              group=[ccc]=symmetry group of molecule,
!
         set +x ;;
#
#     None of the cases have been chosen - flag error.
#
      *)
         echo "%FPX:RUNPRG.SH-WARN, invalid first parameter"
   esac
   exit
