C-----F--1---------2---------3---------4---------5---------6
C
C   MOLSYM.FOR
C
C   20-09-1994: VERSION 1.2 : SYMMETRY COEFFICIENTS FOR A
C   PARTICULAR CLUSTER AND SYMMETRY GROUP. VALID FOR DEC
C   AXP (ALPHA).
C   CONTAINS ROUTINES:
C      MOLSYM (=MAIN)      ORTHOG    PRJECT    TMX
C      DROT      EULER     PERMU
C   NOTE 1: INTERDEPENDANCE:
C      MOLSYM CALLS DROT, EULER, ORTHOG, PERMU, PRJECT, TMX
C   NOTE 2: COMMON BLOCKS:
C      ANGLES    PARAM     ROT       TRIG      XYZ
C      CC        REP
C   NOTE 3: COMMON BLOCK USAGE:
C      MOLSYM USES ANGLES, CC, PARAM, REP, ROT, TRIG, XYZ
C      EULER  USES ANGLES
C      ORTHOG USES CC, PARAM
C      PERMU  USES ANGLES, PARAM, XYZ
C      PRJECT USES CC, PARAM, REP, ROT, TRIG
C      TMX    USES ANGLES, CC, PARAM
C   NOTE 4: SYMBOLIC VARIABLE USAGE:
C      MOLSYM USES #AL,#AT,#BS,#BT,#CL,#GO,#L1,#NL
C      DROT   USES #L1,#LL
C      EULER  USES #GO
C      ORTHOG USES #AT,#BS,#CL,#GO,#L1
C      PERMU  USES #AT,#GO
C      PRJECT USES #AL,#AT,#BS,#BT,#CL,#GO,#L1
C      TMX    USES #AT,#GO
C   NOTE 5: I/O:
C      FOR004    INPUT
C      FOR006    TERMINAL MESSAGES
C      FOR007    OUTPUT OF SYMMETRY COEFFICIENTS
C      FOR009    GENERAL SUMMARY OUTPUT
C
C-----F--1---------2---------3---------4---------5---------6
C
      PROGRAM MOLSYM
C   CALCULATES THE COEFFICIENTS OF THE SPHERICAL HARMONIC
C   BASIS FUNCTIONS THAT TRANSFORM ACCORDING THE IRREDUCIBLE
C   REPRESENTATIONS OF THE SYMMETRY GROUP AT THE VARIOUS
C   ATOMIC SITES IN THE MOLECULAR CLUSTER.
C   ********************************************************
C   BEFORE RUNNING, DIMENSION THE PROGRAM THUS:
C
C      CHANGE '#AL' TO (NUMBER OF ALPHAS AND GAMMAS)
C      CHANGE '#AT' TO (NUMBER OF ATOMS)
C      CHANGE '#BS' TO (NUMBER OF BASIS FUNCTIONS)
C      CHANGE '#BT' TO (NUMBER OF BETAS)
C      CHANGE '#CL' TO (MAX NUMBER OF COLUMNS)
C      CHANGE '#GO' TO (ORDER OF SYMMETRY GROUP)
C      CHANGE '#L1' TO (LMAX+1)
C      CHANGE '#LL' TO (2*LMAX+1)
C      CHANGE '#NL' TO (2*LMAX+1)*(NUMBER OF ATOMS)
C
C   ********************************************************
C   THE GENERAL DIMENSIONING IS AS FOLLOWS:
C
C   ALPHA(NOP),BETA(NOP),GAMMA(NOP),IDEN(NOP)
C   COEF(NAT,LMAX+1,2,NBAS,NCOL)
C   MT(NOP,NAT),MALPH(NOP),MBETA(NOP),MGAM(NOP)
C   DSRY(NOP,NCOL)
C   DPL(LMAX+1,LMAX+1,NBETAS),DMN(LMAX+1,LMAX+1,NBETAS)
C   COSN(LMAX+1,NANGLES),SINN(LMAX+1,NANGLES)
C   ALPHA0(NANGLES),BETA0(NANGLES),MFK(NLX),ICSFK(NLX),
C      NATMFK(NLX),COEFFK(NLX)
C   NATOM(NAT)
C
C   WHERE:
C      LMAX+1=LARGEST L-VALUE PLUS 1
C      NANGLES=NUMBER OF DIFFERENT ALPHA AND GAMMA EULER
C         ANGLES (GENERALLY CAN'T BE FORESEEN, BUT 12
C         SHOULD DO IT)
C      NAT=NUMBER OF ATOMS (INCLUDING OUTER SPHERE)
C      NBAS=NUMBER OF BASES (GENERALLY 10 IS LARGE ENOUGH,
C         BUT MIGHT GO LARGER FOR LARGE LMAX)
C      NBETAS=NUMBER OF DIFFERENT BETA EULER ANGLES (CAN'T
C         BE EASILY FORESEEN, BUT USUALLY 8 IS ENOUGH)
C      NCOL=DIMENSION OF LARGEST IRREP (3 IS MAX)
C      NLX=MAXIMUM OF (2*LXG+1)*MATNG FOR ALL GROUPS OF
C         EQUIVALENT ATOMS, WHERE LXG IS MAXIMUM L-VALUE FOR
C         THE GROUP AND MATNG IS THE MAXIMUM ATOM NUMBER IN
C         THE GROUP
C      NOP=NUMBER OF OPERATIONS IN THE GROUP (#GO IS MAX)
C
C   WARN IS A NEW PARAMETER. IF THE PRODUCT CALCULATED IN
C   THE ORTHOG SUBROUTINE IS LESS THAN WARN, IT'LL FLAG YOU
C   THAT THE CORRESPONDING BASIS FUNCTION MAY NOT REALLY
C   TRANSFORM AS THE IRREDUCIBLE REPRESENTATION.
C
C   FROM CRN <13-3-84. REFORMATTED AND CHECKED 21-10-85 DLF.
C   MOD 23-10-85 DLF : MORE DUMMY DIMENSIONS AND CHANGE IN
C   THE OUTPUT TO CHANNEL 7.
C   MOD 30-6-93 : CHANGE IN INPUT FROM CHANNEL 5.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ALPHAO(#AL),BETAO(#BT),TITLE(20),MFK(#NL),
     & ICSFK(#NL),NATMFK(#NL),COEFFK(#NL)
      DIMENSION WORD(2),NATOM(#AT),ICS(2)
      CHARACTER*10 ID
      CHARACTER*4 IDEN,TITLE,WORD
      LOGICAL CLEAR
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ MT(#GO,#AT),MALPH(#GO),MBETA(#GO),
     & MGAM(#GO),MTAU,KTAU,L,LP1,INVER,MTST,MTX,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ R(3,#AT)
      DATA ICS/1,-1/
      DATA WORD/' COS',' SIN'/
      DATA NGRP,NREP/'GRP ','REP '/
      DATA PI,SQR2/3.141592653589793D0,1.41421356237309D0/
C
      NONE=1
   10 READ(4,3000)LMAX,KTAU,INVER,NREP,MTAU,NEQ,NOUT,
     & NOORTH,IREP,JPUCH,WARN,N7,N8
 3000 FORMAT(10I5,F10.0,I5,I2)
      IF(WARN.EQ.0.0)WARN=1.0
      IF(N7.EQ.1)NONE=2
      WRITE(9,3000)LMAX,KTAU,INVER,NREP,MTAU,NEQ,NOUT,
     & NOORTH,IREP,JPUCH,WARN,N7,N8
      NBETA=0
      NALPH=0
      READ(4,1000)((R(I,M),I=1,3),NATOM(M),M=1,MTAU)
 1000 FORMAT(2(3F10.7,I5,5X))
      WRITE(9,2000)
 2000 FORMAT(//' OPERATION',7X,'AXIS OF ROTATION',8X,
     & 'ANGLE/PI',7X,'(EULER ANGLES)/PI',17X,'PERMUTATION')
C   READING ROTATION ANGLES AND DIRECTIONS TO GENERATE THE
C   EULER ANGLES AND THE PERMUTATIONS OF THE ATOMS.
      DO 90 KT=1,KTAU
      ID(1:6)='      '
      IF(N8.EQ.0)READ(4,1010)ID,OM,EGA,X,Y,Z
 1010 FORMAT(A10,F3.0,3X,F4.3,3F10.7)
      IF(N8.EQ.1)READ(4,1015)ID(7:10),OM,EGA,X,Y,Z
 1015 FORMAT(1X,A4,2F6.2,1X,3F20.16)
      IDEN(KT)=ID(7:10)
      OMEGA=OM/EGA
      CALL EULER(KT)
      WRITE(9,2010)ID,X,Y,Z,OMEGA,ALPHA(KT),BETA(KT),
     & GAMMA(KT)
 2010 FORMAT(A10,7F10.4,8I5)
      ALPHA(KT)=PI*ALPHA(KT)
      BETA(KT)=PI*BETA(KT)
      GAMMA(KT)=PI*GAMMA(KT)
      CALL PERMU(KT)
      WRITE(9,2020)(MT(KT,M),M=1,MTAU)
 2020 FORMAT('+',80X,8I5,9(/,80X,1X,8I5))
      IF(NALPH.EQ.0)GOTO 30
      DO 20 NK=1,NALPH
      IF(DABS(ALPHAO(NK)-ALPHA(KT)).GT.1.0D-4)GOTO 20
      MALPH(KT)=NK
      GOTO 40
   20 CONTINUE
   30 NALPH=NALPH+1
      MALPH(KT)=NALPH
      ALPHAO(NALPH)=ALPHA(KT)
   40 DO 50 NK=1,NALPH
      IF(DABS(ALPHAO(NK)-GAMMA(KT)).GT.1.0D-4)GOTO 50
      MGAM(KT)=NK
      GOTO 60
   50 CONTINUE
      NALPH=NALPH+1
      MGAM(KT)=NALPH
      ALPHAO(NALPH)=GAMMA(KT)
   60 IF(NBETA.EQ.0)GOTO 80
      DO 70 NK=1,NBETA
      IF(DABS(BETAO(NK)-BETA(KT)).GT.1.0D-4)GOTO 70
      MBETA(KT)=NK
      GOTO 90
   70 CONTINUE
   80 NBETA=NBETA+1
      MBETA(KT)=NBETA
      BETAO(NBETA)=BETA(KT)
   90 CONTINUE
      IF(NALPH.GT.#AL.OR.NBETA.GT.#BT)WRITE(9,9000)NALPH,
     & NBETA
 9000 FORMAT(' %MOLSYM-W-DIMERR, check dimensions of DPL,',
     & ' DMN, COSN, SINN, ALPHA0, BETA0'/
     & '  /NALPH=',I3,'  NBETA=',I3,'/')
      IF(NALPH.GT.#AL.OR.NBETA.GT.#BT)WRITE(6,9000)NALPH,
     & NBETA
      CALL TMX(IREP)
      M=0
C   COS AND SIN VALUES FOR THE ALPHA AND GAMMA ROTATIONS.
  100 DO 110 NK=1,NALPH
      COSN(M+1,NK)=DCOS(M*ALPHAO(NK))
  110 SINN(M+1,NK)=DSIN(M*ALPHAO(NK))
      M=M+1
      IF(M.LE.LMAX)GOTO 100
C   RUN THROUGH THE IRREPS TO FIND BASIS FUNCTIONS.
      DO 390 MREP=1,NREP
      NSEC=0
      READ(4,3000)NDIM
      IF(NDIM.EQ.0)NDIM=1
C   INPUT 1ST ROW MATRIX ELEMENTS FOR IRREP.
      DO 120 NCOL=1,NDIM
      READ(4,1020)(TITLE(IIRR),IIRR=1,20)
 1020 FORMAT(20A4)
      WRITE(7,2040)(TITLE(IIRR),IIRR=1,20)
 2040 FORMAT(20A4)
      WRITE(9,2050)(TITLE(IIRR),IIRR=1,20)
 2050 FORMAT(//20A4)
      IF(N8.EQ.0)READ(4,1030)(DSRY(KT,NCOL),KT=1,KTAU)
 1030 FORMAT(8F10.0)
      IF(N8.EQ.1)READ(4,1035)(DSRY(KT,NCOL),KT=1,KTAU)
 1035 FORMAT(4F20.16)
  120 WRITE(9,2060)NCOL,(DSRY(KT,NCOL),KT=1,KTAU)
 2060 FORMAT(60X,'DSRY FOR COL',I3/(1X,8F14.8))
C   RUN THROUGH THE SETS OF EQUIVALENT ATOMS.
      DO 380 MEQ=1,NEQ
      READ(4,3000)MTST,MTX,LSTART,L1
      IF(L1.LE.LMAX)GOTO 130
      WRITE(9,2070)LMAX
 2070 FORMAT(' THE MAXIMUM L FOR THIS ATOMIC GROUP EXCEEDS',
     & ' LMAX.'/'L1 IS SET TO LMAX= ',I5)
      L1=LMAX
  130 LST1=1+LSTART
      LFN1=L1+1
C   FOR A GIVEN GROUP OF ATOMS RUN THROUGH L-VALUES.
      DO 370 LP1=LST1,LFN1
      L=LP1-1
C   GENERATE BETA PART OF D-MATRICES.
      FAC2=1.0D0
      M=0
  140 FAC1=1.0D0
      MP=0
  150 CONTINUE
      FAC=FAC1*FAC2/2.0D0
      DO 170 NK=1,NBETA
      D1=DROT(L,MP,M,BETAO(NK))
      D2=DROT(L,MP,-M,BETAO(NK))
      IF(MOD(M,2).NE.0)D2=-D2
      DPL(MP+1,M+1,NK)=(D1+D2)*FAC
      DMN(MP+1,M+1,NK)=(D1-D2)*FAC
      IF(MOD(M+MP,2).NE.0)GOTO 160
      DPL(M+1,MP+1,NK)=DPL(MP+1,M+1,NK)
      DMN(M+1,MP+1,NK)=DMN(MP+1,M+1,NK)
      GOTO 170
  160 DMN(M+1,MP+1,NK)=-DMN(MP+1,M+1,NK)
      DPL(M+1,MP+1,NK)=-DPL(MP+1,M+1,NK)
  170 CONTINUE
      FAC1=SQR2
      MP=1+MP
      IF(MP.LE.M)GOTO 150
      FAC2=SQR2
      M=1+M
      IF(M.LE.L)GOTO 140
      NBAS=1
C   PROJECT OUT FIRST COLUMN BASIS FUNCTIONS.
      DO 210 MTO=MTST,MTX
      IMAX=1
      M=0
  180 CONTINUE
      DO 200 I=1,IMAX
      CCONS=1.0D0
      CALL PRJECT(M,I,KT,MTO,NBAS,1,CCONS,.TRUE.)
      IF(NOORTH.NE.0)GOTO 190
      CALL ORTHOG(NBAS,WARN)
      IF(NBAS.LE.#BS)GOTO 200
      WRITE(9,9010)NBAS
 9010 FORMAT(' %MOLSYM-W-BASDIM, are the dimensions ',
     & 'adequate ?'/'  /NBAS=',I3,'/')
      WRITE(6,9010)NBAS
      GOTO 200
  190 NBAS=NBAS+1
      IF(NBAS.LE.#BS)GOTO 200
      WRITE(9,9010)NBAS
      WRITE(6,9010)NBAS
  200 CONTINUE
      IMAX=2
      M=M+1
      IF(M.LE.L)GOTO 180
  210 CONTINUE
      NBAS=NBAS-1
      NSEC=NSEC+NBAS
      WRITE(9,2090)NDIM,NBAS
 2090 FORMAT(//40X,'NDIM=',I8,27X,'NBAS=',I8)
      IF(NBAS.EQ.0)GOTO 360
      IF(NDIM.EQ.1)GOTO 290
C   GENERATE COMPANION BASES BY TRANSFER OPERATORS.
      DO 280 NCOL=2,NDIM
      DO 280 NB=1,NBAS
      CLEAR=.TRUE.
      IMAX=1
      DO 240 MTO=MTST,MTX
      M=0
  220 CONTINUE
      DO 230 I=1,IMAX
      AC=COEF(MTO,M+1,I,NB,1)
      IF(DABS(AC).LT.1.0D-4)GOTO 230
      CALL PRJECT(M,I,KT,MTO,NB,NCOL,AC,CLEAR)
      CLEAR=.FALSE.
  230 CONTINUE
      IMAX=2
      M=1+M
      IF(M.LE.L)GOTO 220
  240 CONTINUE
      FAC=DBLE(NDIM)/DBLE(KTAU)
      DO 270 MTO=MTST,MTX
      IMAX=1
      DO 260 M1=1,LP1
      DO 250 I=1,IMAX
      COEF(MTO,M1,I,NB,NCOL)=COEF(MTO,M1,I,NB,NCOL)*FAC
  250 CONTINUE
  260 IMAX=2
  270 CONTINUE
  280 CONTINUE
  290 DO 350 NCOL=1,NDIM
      DO 350 NB=1,NBAS
      IF(NDIM.NE.1)WRITE(9,2100)L,NCOL
 2100 FORMAT(/50X,'L=',I3,'   COLUMN',I3)
      IF(NDIM.EQ.1)WRITE(9,2110)L
 2110 FORMAT(/60X,'L=',I3)
      NMS=0
      M=0
  300 CONTINUE
      IMAX=2
      IF(M.EQ.0)IMAX=1
      DO 340 I=1,IMAX
      SUM=0.0D0
      DO 310 MTO=MTST,MTX
      IF(DABS(COEF(MTO,M+1,I,NB,NCOL)).LT.1.0D-4)
     & COEF(MTO,M+1,I,NB,NCOL)=0.0D0
  310 SUM=SUM+DABS(COEF(MTO,M+1,I,NB,NCOL))
      IF(SUM.LT.1.0D-4)GOTO 340
      WRITE(9,2120)M,WORD(I),
C     & ((COEF(MTO,M+1,I,NB,NCOL),NATOM(MTO)),MTO=MTST,MTX)
C   Modified since g77 seemed to take it as complex.
     & (COEF(MTO,M+1,I,NB,NCOL),NATOM(MTO),MTO=MTST,MTX)
 2120 FORMAT(' M=',I2,A4,'<',8(F11.8,I3),/1X,8(F11.8,I3))
      IF(JPUCH.EQ.2)GOTO 330
      IF((JPUCH.EQ.1).AND.(NCOL.GT.1))GOTO 330
      DO 320 MTFK=MTST,MTX
      MFK(MTFK+NMS)=M
      ICSFK(MTFK+NMS)=ICS(I)
      NATMFK(MTFK+NMS)=NATOM(MTFK)
      COEFFK(MTFK+NMS)=COEF(MTFK,M+1,I,NB,NCOL)
  320 CONTINUE
  330 NMS=NMS+(MTX-MTST+1)
  340 CONTINUE
      M=1+M
      IF(M.LE.L)GOTO 300
      WRITE(9,2130)NMS
 2130 FORMAT(' NMS=',I3)
      IF(JPUCH.EQ.2)GOTO 350
      IF((JPUCH.EQ.1).AND.(NCOL.GT.1))GOTO 350
      NONE=1
      IF(N7.EQ.1)NONE=2
      IF(MTAU.EQ.1)NONE=0
      IF(NMS.NE.0)WRITE(7,3000)L,NMS,NONE
      NFIN=NMS/(MTX-MTST+1)
      MTOP=MTX-MTST+1
      IF(N7.EQ.1)GOTO 345
      WRITE(7,2140)((MFK((IFK-1)*MTOP+JFK),
     & ICSFK((IFK-1)*MTOP+JFK),NATMFK((IFK-1)*MTOP+JFK),
     & COEFFK((IFK-1)*MTOP+JFK),IFK=1,NFIN),JFK=MTST,MTX)
 2140 FORMAT(4(2I2,I4,F12.9))
      GOTO 350
  345 WRITE(7,2145)((MFK((IFK-1)*MTOP+JFK),
     & ICSFK((IFK-1)*MTOP+JFK),NATMFK((IFK-1)*MTOP+JFK),
     & COEFFK((IFK-1)*MTOP+JFK),IFK=1,NFIN),JFK=MTST,MTX)
 2145 FORMAT(2(5X,2I3,I5,E24.16))
  350 CONTINUE
      GOTO 370
  360 WRITE(9,2150)L
 2150 FORMAT(//40X,'NO  BASIS  FOUND  FOR  L= ',I3/)
  370 CONTINUE
  380 CONTINUE
      WRITE(9,2160)NSEC
 2160 FORMAT(' DIMENSION OF THE SECULAR DETERMINANT= ',I3)
      WRITE(6,2170)TITLE(1),TITLE(2),NSEC
 2170 FORMAT('    IRREP ',2A4,',  DIM. SEC. MAT. = ',I3)
  390 CONTINUE
      READ(4,1020)NGO
      IF(NGO.EQ.NGRP)GOTO 10
      STOP 12040
      END
C-----F--1---------2---------3---------4---------5---------6
C
      FUNCTION DROT(L,MP,M,BETA)
C   RETURNS THE PART OF THE D COEFFICIENT RELATED TO THE
C   BETA EULER ANGLE. THE EQUATION (4.13) OF M.E.ROSE (1957,
C   "ELEMENTARY THEORY OF ANGULAR MOMENTUM", WILEY, NEW
C   YORK) IS USED.
C   WRITTEN 11-10-85 DLF.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION NINC(#L1),RATC(#L1),RATS(#L1)
      DIMENSION FI(#LL),SQ(#LL)
      DIMENSION AM(4),MF(4),NF(4)
      EQUIVALENCE (M1,MF(1)),(M2,MF(2)),(M3,MF(3)),
     & (M4,MF(4))
      EQUIVALENCE (N1,NF(1)),(N2,NF(2)),(N3,NF(3)),
     & (N4,NF(4))
      DATA NSET/0/,LL/#LL/
C
      IF(NSET.EQ.1)GOTO 20
      DO 10 I=1,LL
      FI(I)=DBLE(I)
   10 SQ(I)=DSQRT(FI(I))
      NSET=1
   20 IF(L.EQ.0)GOTO 180
      BETA2=0.5D0*BETA
      COSB=DCOS(BETA2)
      SINB=-DSIN(BETA2)
      C2=COSB**2
      S2=1.0D0-C2
      N1=L+M
      N2=L-M
      N3=L+MP
      N4=L-MP
      MPMM=MP-M
      KMAX=MIN0(N4,N1)
      KMIN=MAX0(0,-MPMM)
      NKM1=KMAX-KMIN
      NK=NKM1+1
      KMINM1=KMIN-1
      IF(NKM1.EQ.0)GOTO 80
      DO 60 I=1,NKM1
      K=I+KMINM1
      KP1=K+1
      RATC(I)=C2*(MPMM+KP1)*KP1
      RATS(I)=S2*(N4-K)*(N1-K)
      IF(RATS(I)-RATC(I))30,40,50
   30 NINC(I)=-1
      GOTO 60
   40 NINC(I)=0
      GOTO 60
   50 NINC(I)=1
   60 CONTINUE
      DO 70 I=1,NKM1
      IF(NINC(I).LT.1)GOTO 90
   70 CONTINUE
   80 I=NK
   90 KTOP=I+KMINM1
      M1=N4-KTOP
      M2=N1-KTOP
      M3=KTOP+MPMM
      M4=KTOP
      AM(1)=COSB
      AM(2)=COSB
      AM(3)=SINB
      AM(4)=SINB
      TERM=1.0D0
      IF(MOD(KTOP,2).NE.0)TERM=-1.0D0
      LD=2*L
      NA=1
      KA=0
      NB=3
      KB=0
      NC=1
      KC=0
      KL=0
  100 KL=KL+1
  110 KA=KA+1
      IF(KA.LE.NF(NA))GOTO 120
      NA=NA+1
      KA=0
      GOTO 110
  120 KB=KB+1
      IF(KB.LE.NF(NB))GOTO 130
      NB=NB+1
      KB=0
      GOTO 120
  130 KC=KC+1
      IF(KC.LE.MF(NC))GOTO 140
      NC=NC+1
      KC=0
      GOTO 130
  140 TERM=TERM*((SQ(KA)*SQ(KB)*AM(NC))/FI(KC))
      IF(KL.LT.LD)GOTO 100
      DROT=TERM
      K0=KTOP-KMINM1
      IF(KTOP.EQ.KMAX)GOTO 160
      UP=TERM
      K=K0
  150 J=K
      K=K+1
      UP=-UP*(RATS(J)/RATC(J))
      DROT=DROT+UP
      IF(K.LT.NK)GOTO 150
  160 IF(KTOP.EQ.KMIN)RETURN
      DOWN=TERM
      K=K0
  170 K=K-1
      DOWN=-DOWN*(RATC(K)/RATS(K))
      DROT=DROT+DOWN
      IF(K.GT.1)GOTO 170
      RETURN
  180 DROT=1.0D0
      RETURN
      END
C-----F--1---------2---------3---------4---------5---------6
C
      SUBROUTINE EULER(KT)
C   FINDS THE EULER ANGLES FOR THE GROUP ELEMENT KT. USES
C   THE "Y-CONVENTION" OF H. GOLDSTEIN (1980, "CLASSICAL
C   MECHANICS", 2ND EDN., ADDISON-WESLEY, READING, MASS.,
C   CH. 4 AND APP. B). THE OPERATIONS ARE PERFORMED AS
C   FOLLOWS: FIRST ROTATE ANGLE GAMMA ABOUT Z-AXIS, THEN
C   BETA ABOUT INTERMEDIATE Y-AXIS, THEN ALPHA ABOUT FINAL
C   Z-AXIS.
C   FROM CRN <13-3-84. REFORMATTED AND CHECKED 10-10-85 DLF.
C   MOD 23-10-85 DLF : DUMMY DIMENSIONED.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE COMPLEX ALF,BET,AB,BA
      CHARACTER*4 IDEN
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ MT(#GO,#AT),MALPH(#GO),MBETA(#GO),
     & MGAM(#GO),MTAU,KTAU,L,LP1,INVER,MTST,MTX,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ RRR(3,#AT)
      DATA PI/3.141592653589793D0/
C
      OMEGA=DMOD(OMEGA,2.0D0)*PI
      ALPHA(KT)=0.0D0
      BETA(KT)=0.0D0
      GAMMA(KT)=0.0D0
      IF(OMEGA.EQ.0.0D0)RETURN
      R=DSQRT(X*X+Y*Y+Z*Z)
      IF(R.EQ.0.0D0)GOTO 9000
      X=X/R
      Y=Y/R
      Z=Z/R
      P=DSQRT(X*X+Y*Y)
      IF(P.EQ.0.0D0)GOTO 50
      IF(Z.EQ.0.0D0)GOTO 60
      C=DCOS(0.5D0*OMEGA)
      S=DSIN(0.5D0*OMEGA)
      STH=P*S
      CTH=DSQRT(1.0D0-STH**2)
      SC=STH*CTH
      IF(DABS(SC).LT.1.0D-3)GOTO 9000
C   EULER PARAMETERS E0=C, E1=X*S, E2=Y*S AND E3=Z*S.
C   PREPARE CAYLEY-KLEIN PARAMETERS ALF AND BET.
      ALF=DCMPLX(C,Z*S)
      BET=DCMPLX(Y*S,X*S)
      AB=ALF*BET/SC
      BA=ALF*DCONJG(BET)/SC
      CA=DREAL(BA)
      SA=DIMAG(BA)
      CG=DREAL(AB)
      SG=DIMAG(AB)
      UA=1.0D0+CA
      UG=1.0D0+CG
      IF(DABS(UA).LT.1.0D-14)UA=0.0D0
      IF(DABS(UG).LT.1.0D-14)UG=0.0D0
      IF(UA.EQ.0.0D0.AND.SA.EQ.0.0D0)GOTO 10
      ALPHA(KT)=2.0D0*DATAN2(SA,UA)/PI
      GOTO 20
   10 ALPHA(KT)=1.0D0
   20 IF(UG.EQ.0.0D0.AND.SG.EQ.0.0D0)GOTO 30
      GAMMA(KT)=2.0D0*DATAN2(SG,UG)/PI
      GOTO 40
   30 GAMMA(KT)=1.0D0
   40 V=STH
      IF(DABS(V).GT.1.0D0)V=DSIGN(1.0D0,STH)
      BETA(KT)=2.0D0*DASIN(V)/PI
      OMEGA=OMEGA/PI
      X=X*R
      Y=Y*R
      Z=Z*R
      RETURN
C   ONLY ROTATE ABOUT Z-AXIS.
   50 GAMMA(KT)=Z*OMEGA/PI
      OMEGA=OMEGA/PI
      Z=Z*R
      RETURN
C   ROTATION AXIS IN XY-PLANE.
   60 AB=DCMPLX(Y,X)
      BA=DCMPLX(Y,-X)
      CA=DREAL(BA)
      SA=DIMAG(BA)
      CG=DREAL(AB)
      SG=DIMAG(AB)
      UA=1.0D0+CA
      UG=1.0D0+CG
      IF(DABS(UA).LT.1.0D-14)UA=0.0D0
      IF(DABS(UG).LT.1.0D-14)UG=0.0D0
      IF(UA.EQ.0.0D0.AND.SA.EQ.0.0D0)GOTO 70
      ALPHA(KT)=2.0D0*DATAN2(SA,UA)/PI
      GOTO 80
   70 ALPHA(KT)=1.0D0
   80 IF(UG.EQ.0.0D0.AND.SG.EQ.0.0D0)GOTO 90
      GAMMA(KT)=2.0D0*DATAN2(SG,UG)/PI
      GOTO 100
   90 GAMMA(KT)=1.0D0
  100 CONTINUE
      BETA(KT)=OMEGA/PI
      OMEGA=OMEGA/PI
      X=X*R
      Y=Y*R
      RETURN
 9000 WRITE(9,9010)R,STH,CTH
 9010 FORMAT(' %MOLSYM-F-BADINP, faulty input data'/
     & '  /R=',E14.7,'  STH=',E14.7,'  CTH=',E14.7,'/')
      WRITE(6,9010)R,STH,CTH
      STOP 12049
      END
C-----F--1---------2---------3---------4---------5---------6
C
      SUBROUTINE ORTHOG(NBAS,WARN)
C   ORTHOGONALISES A NEW CANDIDATE BASIS FUNCTION WITH
C   RESPECT TO ALL PREVIOUS FUNCTIONS. IF RESULT IS LARGE
C   ENOUGH THEN ADMITS IT AS BASIS FUNCTION AND NORMALISES.
C   FROM CRN <13-3-85. REFORMATTED AND CHECKED 21-10-85 DLF.
C   MOD 23-10-85 DLF : MORE DUMMY DIMENSIONS.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ NDUMMY(#GO,#AT),NDUN1(#GO),
     & NDUN2(#GO),NDUN3(#GO),MTAU,
     & KTAU,L,LP1,IDUMMY,MTST,MTX,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ R(3,#AT)
C
      IF(NBAS.EQ.1)GOTO 60
C   ORTHOGONALISATION USING GRAM-SCHMIDT PROCEDURE.
      NN1=NBAS-1
      DO 50 N=1,NN1
      PROD=0.0D0
      IMAX=1
      DO 20 M1=1,LP1
      DO 10 I=1,IMAX
      DO 10 MT=MTST,MTX
   10 PROD=PROD+COEF(MT,M1,I,NBAS,1)*COEF(MT,M1,I,N,1)
   20 IMAX=2
      IMAX=1
      DO 40 M1=1,LP1
      DO 30 I=1,IMAX
      DO 30 MT=MTST,MTX
   30 COEF(MT,M1,I,NBAS,1)=COEF(MT,M1,I,NBAS,1)-
     & PROD*COEF(MT,M1,I,N,1)
   40 IMAX=2
   50 CONTINUE
C   NORMALISATION.
   60 PROD=0.0D0
      IMAX=1
      DO 80 M1=1,LP1
      DO 70 I=1,IMAX
      DO 70 MT=MTST,MTX
   70 PROD=PROD+COEF(MT,M1,I,NBAS,1)**2
   80 IMAX=2
      IF(PROD.LE.1.0D-4) RETURN
      IF(PROD.GT.WARN)GOTO 90
      WRITE(9,2000)
 2000 FORMAT(//,20X,'**********WARNING**********')
      WRITE(9,2010)NBAS
 2010 FORMAT(//,20X,'BASIS NUMBER',I3,' IS BELOW WARN ',
     & 'PARAMETER')
      WRITE(9,2020)PROD
 2020 FORMAT(/,30X,'PROD =',1PE13.5)
      WRITE(9,2030)
 2030 FORMAT(//,20X,'***************************',//)
   90 PROD=DSQRT(PROD)
      IMAX=1
      DO 110 M1=1,LP1
      DO 100 I=1,IMAX
      DO 100 MT=MTST,MTX
  100 COEF(MT,M1,I,NBAS,1)=COEF(MT,M1,I,NBAS,1)/PROD
  110 IMAX=2
      NBAS=NBAS+1
      RETURN
      END
C-----F--1---------2---------3---------4---------5---------6
C
      SUBROUTINE PERMU(KT)
C   PERMUTES THE ATOMIC POSITIONS USING A GROUP ELEMENT.
C   FROM CRN <13-3-84. REFORMATTED AND CHECKED 10-10-85 DLF.
C   MOD 23-10-85 DLF : MORE DUMMY DIMENSIONS.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*4 IDEN
      DIMENSION A(3,3),S(3)
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ MT(#GO,#AT),MALPH(#GO),MBETA(#GO),
     & MGAM(#GO),MTAU,KTAU,L,LP1,INVER,MDUM1,MDUM2,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ R(3,#AT)
C
C   COS AND SIN OF EULER ANGLES FOR GROUP ELEMENT KT.
      CA=DCOS(ALPHA(KT))
      SA=DSIN(ALPHA(KT))
      CB=DCOS(BETA(KT))
      SB=DSIN(BETA(KT))
      CG=DCOS(GAMMA(KT))
      SG=DSIN(GAMMA(KT))
C   ROTATION MATRIX A.
      A(1,1)=CA*CB*CG-SA*SG
      A(2,1)=SA*CB*CG+CA*SG
      A(3,1)=-SB*CG
      A(1,2)=-CA*CB*SG-SA*CG
      A(2,2)=-SA*CB*SG+CA*CG
      A(3,2)=SB*SG
      A(1,3)=CA*SB
      A(2,3)=SA*SB
      A(3,3)=CB
C   APPLYING ROTATION TO ATOMIC POSITIONS.
      DO 40 M=1,MTAU
      DO 10 I=1,3
      S(I)=0.0D0
      DO 10 J=1,3
   10 S(I)=S(I)+A(I,J)*R(J,M)
      IF(KT.LE.INVER)GOTO 20
      S(1)=-S(1)
      S(2)=-S(2)
      S(3)=-S(3)
   20 MP1=1
      IF((NOUT.EQ.1).AND.(M.EQ.2))MP1=2
      DO 30 MP=MP1,MTAU
      RS=DABS(R(1,MP)-S(1))+DABS(R(2,MP)-S(2))+
     & DABS(R(3,MP)-S(3))
      IF(RS.LT.1.0D-2)GOTO 40
   30 CONTINUE
      WRITE(9,9000)KT,M,S(1),S(2),S(3)
 9000 FORMAT(' %MOLSYM-F-PERMUT, stop in permute'/
     & '  /KT=',I5,'  M=',I5,'  S(1)=',E14.7,'  S(2)=',
     & E14.7,'  S(3)=',E14.7,'/')
      WRITE(6,9000)KT,M,S(1),S(2),S(3)
      STOP 12049
   40 MT(KT,M)=MP
      RETURN
      END
C-----F--1---------2---------3---------4---------5---------6
C
      SUBROUTINE PRJECT(M,I,KT,MTO,NBAS,NCOL,CONST,CLEAR)
C   APPLIES THE PROJECTION OPERATOR TO YL(L,M,I).
C   FROM CRN <13-3-84. REFORMATTED AND CHECKED 21-10-85 DLF.
C   MOD 23-10-85 DLF : MORE DUMMY DIMENSIONS.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      LOGICAL CLEAR
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ MT(#GO,#AT),MALPH(#GO),MBETA(#GO),
     & MGAM(#GO),MTAU,KTAU,L,LP1,INVER,MTST,MTX,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ R(3,#AT)
C
      IF(.NOT.CLEAR)GOTO 20
      DO 10 MT1=MTST,MTX
      DO 10 M1=1,LP1
      DO 10 I1=1,2
      COEF(MT1,M1,I1,NBAS,NCOL)=0.0D0
   10 CONTINUE
C   PROJECTION OF YL(L,M,I).
   20 IPMAX=1
      LOV=L+1
      DO 90 MQ=1,LOV
      MP=MQ-1
      DO 80 IP=1,IPMAX
      DO 70 KT=1,KTAU
      MAKT=MALPH(KT)
      MBKT=MBETA(KT)
      MGKT=MGAM(KT)
      IF(I.EQ.2)GOTO 30
      IF(IP.EQ.2)GOTO 50
      D= COSN(MP+1,MAKT)*COSN(M+1,MGKT)*DPL(MP+1,M+1,MBKT)
     &  -SINN(MP+1,MAKT)*SINN(M+1,MGKT)*DMN(MP+1,M+1,MBKT)
      GOTO 60
   30 IF(IP.EQ.2)GOTO 40
      D=-COSN(MP+1,MAKT)*SINN(M+1,MGKT)*DPL(MP+1,M+1,MBKT)
     &  -SINN(MP+1,MAKT)*COSN(M+1,MGKT)*DMN(MP+1,M+1,MBKT)
      GOTO 60
   40 D=-SINN(MP+1,MAKT)*SINN(M+1,MGKT)*DPL(MP+1,M+1,MBKT)
     &  +COSN(MP+1,MAKT)*COSN(M+1,MGKT)*DMN(MP+1,M+1,MBKT)
      GOTO 60
   50 D= SINN(MP+1,MAKT)*COSN(M+1,MGKT)*DPL(MP+1,M+1,MBKT)
     &  +COSN(MP+1,MAKT)*SINN(M+1,MGKT)*DMN(MP+1,M+1,MBKT)
   60 IF(KT.GT.INVER.AND.MOD(L,2).NE.0)D=-D
   70 COEF(MT(KT,MTO),MP+1,IP,NBAS,NCOL)=
     & COEF(MT(KT,MTO),MP+1,IP,NBAS,NCOL)+
     & DSRY(KT,NCOL)*D*CONST
   80 CONTINUE
   90 IPMAX=2
      RETURN
      END
C-----F--1---------2---------3---------4---------5---------6
C
      SUBROUTINE TMX(IREP)
C   GENERATES ROTATION MATRICES FOR GROUP ELEMENTS. IF
C   IREP=0 THEN NO MATRICES GENERATED; IF IREP=1,-1 THEN
C   3-DIM. MATRICES GENERATED; IF IREP=2,-2 THEN 3-DIM. AND
C   5-DIM. MATRICES GENERATED. FOR IREP<0 THE PROGRAM IS
C   STOPPED.
C   FROM CRN <13-3-84. REFORMATTED AND CHECKED 10-10-85 DLF.
C   MOD 23-10-85 DLF : MORE DUMMY DIMENSIONS.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*4 IDEN
      DIMENSION RA(5,5),RB(5,5),RG(5,5),T(5,5,#GO)
      COMMON /ANGLES/ ALPHA(#GO),BETA(#GO),GAMMA(#GO),OMEGA,
     & X,Y,Z
      COMMON /ANG2/ IDEN(#GO)
      COMMON /CC/ COEF(#AT,#L1,2,#BS,#CL)
      COMMON /PARAM/ MT(#GO,#AT),MALPH(#GO),MBETA(#GO),
     & MGAM(#GO),MTAU,KTAU,L,LP1,INVER,MDUMI1,MDUMI2,NOUT
      COMMON /REP/ DSRY(#GO,#CL)
      COMMON /ROT/ DPL(#L1,#L1,#BT),DMN(#L1,#L1,#BT)
      COMMON /TRIG/ COSN(#L1,#AL),SINN(#L1,#AL)
      COMMON /XYZ/ R(3,#AT)
C
      IF(IREP.EQ.0)RETURN
C   GENERATING 3-DIM. MATRICES.
      DO 10 KT=1,KTAU
      S=1.0D0
      IF(KT.GT.INVER)S=-1.0D0
      CA=DCOS(ALPHA(KT))
      SA=DSIN(ALPHA(KT))
      CB=DCOS(BETA(KT))
      SB=DSIN(BETA(KT))
      CG=DCOS(GAMMA(KT))
      SG=DSIN(GAMMA(KT))
      T(1,1,KT)=(CA*CB*CG-SA*SG)*S
      T(1,2,KT)=(SA*CB*CG+CA*SG)*S
      T(1,3,KT)=-SB*CG*S
      T(2,1,KT)=(-CA*CB*SG-SA*CG)*S
      T(2,2,KT)=(-SA*CB*SG+CA*CG)*S
      T(2,3,KT)=SB*SG*S
      T(3,1,KT)=CA*SB*S
      T(3,2,KT)=SA*SB*S
   10 T(3,3,KT)=CB*S
      DO 30 KT=1,KTAU,6
      KTM=KT-1
      WRITE(9,2000)(IDEN(KTM+K),K=1,6)
 2000 FORMAT(//2X,6(6X,A4,10X))
      DO 20 J=1,3
   20 WRITE(9,2010)((T(I,J,KTM+K),I=1,3),K=1,6)
 2010 FORMAT('  ',6('*',3F6.3,'*'))
   30 CONTINUE
      IF(IREP.EQ.1)RETURN
      IF(IREP.EQ.-1)GOTO 9000
C   GENERATING 5-DIM. MATRICES.
      RT=DSQRT(3.0D0)
      DO 60 KT=1,KTAU
      DO 40 I=1,5
      DO 40 J=1,5
      RA(I,J)=0.0D0
      RB(I,J)=0.0D0
      RG(I,J)=0.0D0
   40 T(I,J,KT)=0.0D0
      CA=DCOS(ALPHA(KT))
      SA=DSIN(ALPHA(KT))
      CB=DCOS(BETA(KT))
      SB=DSIN(BETA(KT))
      CG=DCOS(GAMMA(KT))
      SG=DSIN(GAMMA(KT))
      RA(1,1)=1.0D0
      RA(2,2)=2.0D0*CA*CA-1.0D0
      RA(2,3)=2.0D0*CA*SA
      RA(3,2)=-RA(2,3)
      RA(3,3)=RA(2,2)
      RA(4,4)=CA
      RA(4,5)=SA
      RA(5,4)=-SA
      RA(5,5)=CA
      RG(1,1)=1.0D0
      RG(2,2)=2.0D0*CG*CG-1.0D0
      RG(2,3)=2.0D0*CG*SG
      RG(3,2)=-RG(2,3)
      RG(3,3)=RG(2,2)
      RG(4,4)=CG
      RG(4,5)=SG
      RG(5,4)=-SG
      RG(5,5)=CG
      RB(2,2)=0.5D0*CB*CB+0.5D0
      RB(1,1)=3.0D0*RB(2,2)-2.0D0
      RB(4,4)=4.0D0*RB(2,2)-3.0D0
      RB(3,3)=CB
      RB(3,5)=-SB
      RB(5,3)=SB
      RB(4,2)=CB*SB
      RB(2,4)=-RB(4,2)
      RB(1,4)=RB(4,2)*RT
      RB(4,1)=-RB(1,4)
      RB(5,5)=CB
      RB(1,2)=0.5D0*RT*SB*SB
      RB(2,1)=RB(1,2)
      DO 50 I=1,5
      DO 50 J=1,5
      DO 50 K=1,5
      DO 50 L=1,5
   50 T(I,J,KT)=T(I,J,KT)+RG(I,K)*RB(K,L)*RA(L,J)
   60 CONTINUE
      DO 80 KT=1,KTAU,3
      KTM=KT-1
      WRITE(9,2020)(IDEN(KTM+K),K=1,3)
 2020 FORMAT(//2X,3(13X,A4,13X))
      DO 70 J=1,5
   70 WRITE(9,2030)((T(I,J,KTM+K),I=1,5),K=1,3)
 2030 FORMAT('  ',3('*',F6.3,4F7.3,'*'))
   80 CONTINUE
      IF(IREP.EQ.2)RETURN
 9000 WRITE(9,9010)IREP
 9010 FORMAT(' %MOLSYM-F-TMXSTP, stop in TMX - IREP=',I6)
      WRITE(6,9010)IREP
      STOP 12049
      END
